==> kolo-1.3.8.tar.gz: kolo-1.3.8/src/kolo/middleware.py <==
import logging
import os
import sys
import threading
from typing import Any, Callable, Dict, Tuple

from django.conf import settings
from django.http import HttpRequest, HttpResponse

from .config import create_kolo_directory, load_config_from_toml
from .db import load_config_from_db, setup_db
from .profiler import KoloProfiler

logger = logging.getLogger("kolo")

DjangoView = Callable[[HttpRequest], HttpResponse]


class KoloMiddleware:
    def __init__(self, get_response: DjangoView) -> None:
        self.get_response = get_response
        self.enabled = self.should_enable()
        if self.enabled:
            kolo_directory = create_kolo_directory()
            self.config = load_config_from_toml(kolo_directory / "config.toml")
            self.db_path = setup_db(self.config)

    def __call__(self, request: HttpRequest) -> HttpResponse:
        if not self.enabled:
            return self.get_response(request)

        if self.check_for_third_party_profiler():
            return self.get_response(request)

        self.config = load_config_from_db(self.db_path, self.config)

        filter_config = self.config.get("filters", {})
        ignore_request_paths = filter_config.get("ignore_request_paths", [])
        for path in ignore_request_paths:
            if path in request.path:
                return self.get_response(request)

        self.profiler = KoloProfiler(
            self.db_path,
            include_frames=filter_config.get("include_frames", []),
            ignore_frames=filter_config.get("ignore_frames", []),
            config=self.config,
        )
        self.profiler.initialize_request(request)

        with self.profiler:
            response = self.get_response(request)

        self.profiler.finalize_response(response)

        threading.Thread(target=self.profiler.save_request_in_db).start()

        # eventual todo: restore previously set profile
        return response

    def process_view(
        self,
        request: HttpRequest,
        view_func: DjangoView,
        view_args: Tuple[Any],
        view_kwargs: Dict[str, Any],
    ):
        if not self.enabled:
            return

        self.profiler.set_url_pattern(request)

    def check_for_third_party_profiler(self) -> bool:
        profiler = sys.getprofile()
        if profiler:
            logger.warning("Profiler %s is active, disabling KoloMiddleware", profiler)
            return True
        return False

    def should_enable(self) -> bool:
        if settings.DEBUG is False:
            logger.debug("DEBUG mode is off, disabling KoloMiddleware")
            return False

        if os.environ.get("KOLO_DISABLE", "false").lower() not in ["false", "0"]:
            logger.debug("KOLO_DISABLE is set, disabling KoloMiddleware")
            return False

        if self.check_for_third_party_profiler():
            return False

        return True

==> kolo-1.8.0.tar.gz: kolo-1.8.0/src/kolo/middleware.py <==
import logging
import os
import sys
import threading
from typing import Callable

from django.conf import settings
from django.http import HttpRequest, HttpResponse

from .config import create_kolo_directory, load_config_from_toml
from .db import load_config_from_db, setup_db
from .profiler import KoloProfiler

logger = logging.getLogger("kolo")

DjangoView = Callable[[HttpRequest], HttpResponse]


class KoloMiddleware:
    def __init__(self, get_response: DjangoView) -> None:
        self._get_response = get_response
        self.enabled = self.should_enable()
        if self.enabled:
            kolo_directory = create_kolo_directory()
            self.config = load_config_from_toml(kolo_directory / "config.toml")
            self.db_path = setup_db(self.config)

    def __call__(self, request: HttpRequest) -> HttpResponse:
        # WARNING: Because Django's runserver uses threading, we need
        # to be careful about thread safety here.
        if not self.enabled or self.check_for_third_party_profiler():
            return self.get_response(request)

        # This technically *isn't* threadsafe because saving a new config
        # here will overwrite it for other threads, but we don't actually
        # care. A different thread getting a newer config is acceptable.
        self.config = load_config_from_db(self.db_path, self.config)

        filter_config = self.config.get("filters", {})
        ignore_request_paths = filter_config.get("ignore_request_paths", [])
        for path in ignore_request_paths:
            if path in request.path:
                return self.get_response(request)

        # Don't store the KoloProfiler on self to avoid threadsafety
        # bugs. If a different thread gets this instance of KoloProfiler
        # at the wrong time, we lose the original profiler's trace.
        profiler = KoloProfiler(self.db_path, config=self.config)

        with profiler:
            response = self.get_response(request)

        name = "kolo-save_request_in_db"
        threading.Thread(target=profiler.save_request_in_db, name=name).start()
        return response

    def get_response(self, request: HttpRequest) -> HttpResponse:
        response = self._get_response(request)
        return response

    def check_for_third_party_profiler(self) -> bool:
        profiler = sys.getprofile()
        if profiler:
            logger.warning("Profiler %s is active, disabling KoloMiddleware", profiler)
            return True
        return False

    def should_enable(self) -> bool:
        if settings.DEBUG is False:
            logger.debug("DEBUG mode is off, disabling KoloMiddleware")
            return False

        if os.environ.get("KOLO_DISABLE", "false").lower() not in ["false", "0"]:
            logger.debug("KOLO_DISABLE is set, disabling KoloMiddleware")
            return False

        if self.check_for_third_party_profiler():
            return False

        return True
